package net.wanji.com.service.controller.impl;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.com.cache.CrossInfoCache;
import net.wanji.com.common.constants.Constants;
import net.wanji.com.common.enums.ControlModelEnum;
import net.wanji.com.common.enums.SectionControlModeEnum;
import net.wanji.com.netty.NettyClient;
import net.wanji.com.pojo.netty.MessageResultPojo;
import net.wanji.com.service.controller.ControlCommandService;
import net.wanji.com.util.CommonUtils;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.vo.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 10:43
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class DTControlCommandServiceImpl implements ControlCommandService {

    private final CrossInfoCache crossInfoCache;

    @Override
    public String getBeanMark() {
        return BaseEnum.VendorTypeEnum.DT.getNick();
    }

    @Override
    public JsonViewObject schemeSend(SchemeSendVO schemeSendVO) throws Exception {
        String crossId = schemeSendVO.getCrossCode();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        List<SchemeSendVO.Pattern> schemeVOList = schemeSendVO.getPatternList();
        setSchemePhaseStageInfo(crossInfoPO, schemeVOList);
        return JsonViewObject.newInstance().success("下发成功异常");
    }

    private void setSchemePhaseStageInfo(CrossInfoPO crossInfoPO, List<SchemeSendVO.Pattern> schemeVOList) {
        setPhaseStageInfo(crossInfoPO, schemeVOList);
        setSchemeInfo(crossInfoPO, schemeVOList);

    }

    private void setSchemeInfo(CrossInfoPO crossInfoPO, List<SchemeSendVO.Pattern> schemeVOList) {
        for (SchemeSendVO.Pattern schemeVO : schemeVOList) {
            String schemeNo = schemeVO.getPatternNo();
            String cycle = schemeVO.getCycle();
            String offset = schemeVO.getOffset();
            List<SchemeSendVO.Pattern.Ring> rings = schemeVO.getRings();
            for (SchemeSendVO.Pattern.Ring ring : rings) {
                String ringNo = ring.getRingNo();
                List<SchemeSendVO.Pattern.Ring.Phase> phaseList = ring.getPhaseList();
            }
        }

    }

    private void setPhaseStageInfo(CrossInfoPO crossInfoPO, List<SchemeSendVO.Pattern> schemeVOList) {

    }


    @Override
    public JsonViewObject planSend(PlanSendVO planSendVO) throws Exception {
        String crossId = planSendVO.getCrossCode();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        List<PlanSendVO.Plan> planList = planSendVO.getPlanList();
        StringBuilder sb = new StringBuilder();
        for (PlanSendVO.Plan plan : planList) {
            sb.append(buidDailyPlanHex(plan));
        }
        // 只下发有效计划返回报错
        if (planList.size() < 8) {
            for (int i = planList.size() + 1; i <= 8 - planList.size(); i++) {
                PlanSendVO.Plan plan = new PlanSendVO.Plan();
                plan.setPlanNo(String.valueOf(i));
                sb.append(buidDailyPlanHex(plan));
            }
        }
        String bodyValueLength = String.format("%02x", planList.size() * 5); // 调度数量 * 属性数量
        String message = String.format(Constants.COMMAND_SET_COMMON, Constants.COMMAND_SET, bodyValueLength, sb.substring(0, sb.length() - 4));
        String planSign = StringUtils.join(Constants.COMMAND_DAILYPLAN_CONFIG, Constants.COMMAND_SET_REPLY);
        String planMessageHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), planMessageHex, planSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            return JsonViewObject.newInstance().success("下发计划成功");
        }
        return JsonViewObject.newInstance().success("下发计划异常");
    }

    private String buidDailyPlanHex(PlanSendVO.Plan plan) {
        String planNo = plan.getPlanNo();
        List<PlanSendVO.Plan.Section> sectionList = plan.getSectionList();
        StringBuilder hourMinuteSb = new StringBuilder();
        StringBuilder schemeNoSb = new StringBuilder();
        StringBuilder controlModelSb = new StringBuilder();
        if (!CollectionUtils.isEmpty(sectionList)) {
            for (PlanSendVO.Plan.Section section : sectionList) {
                String schemeNo = section.getPatternNo();
                String schemeHex = String.format("%02x", Integer.valueOf(schemeNo));
                schemeNoSb.insert(0, schemeHex).insert(0, schemeHex); // 开始时间结束时间两个时间段，插入两次
                String beginTime = section.getBeginTime();
                String startHourMinuteHex = CommonUtils.getHourMinuteHex(beginTime);
                String endTime = section.getEndTime();
                String endHourMinuteHex = CommonUtils.getHourMinuteHex(endTime);
                hourMinuteSb.insert(0, startHourMinuteHex).insert(0, endHourMinuteHex);
                String controlMode = section.getControlMode();
                String dtControlMode = SectionControlModeEnum.getDtControlMode(Integer.valueOf(controlMode));
                controlModelSb.insert(0, dtControlMode).insert(0, dtControlMode);
            }
        }
        String matchStr = "0b02%s%s";
        StringBuffer sb = new StringBuffer();
        for (int i = 2; i <= 5; i++) { // 属性
            sb.append(String.format(matchStr, String.format("%02x", i), String.format("%02x", Integer.parseInt(planNo))));
            if (i == 2) {
                sb.append("01").append(String.format("%02x", i)).append("64");
            }
            if (i == 3) {
                sb.append(StringUtils.leftPad(hourMinuteSb.toString(), 192, "0")).append(String.format("%02x", i)).append("34");
            }
            if (i == 4) {
                sb.append(StringUtils.leftPad(schemeNoSb.toString(), 96, "0")).append(String.format("%02x", i)).append("34");
            }
            if (i == 5) {
                sb.append(StringUtils.leftPad(controlModelSb.toString(), 96, "0")).append(String.format("%02x", i)).append("05");
            }
        }
        return sb.toString();
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception {
        String crossId = scheduleSendVO.getCrossCode();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        List<ScheduleSendVO.Schedule> schedules = scheduleSendVO.getSchedules();
        StringBuilder sb = new StringBuilder();
        for (ScheduleSendVO.Schedule schedule : schedules) {
            sb.append(buildScheduleHex(schedule));
        }
        String bodyValueLeght = String.format("%02x", schedules.size() * 6); // 调度数量 * 属性数量
        String message = String.format(Constants.COMMAND_SET_COMMON, Constants.COMMAND_SET, bodyValueLeght, sb.toString());
        String scheduleSign = StringUtils.join("0901", Constants.COMMAND_SET_REPLY);
        String scheduleMessage = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), scheduleMessage, scheduleSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            return JsonViewObject.newInstance().success("下发调度成功");
        }
        return JsonViewObject.newInstance().success("下发调度异常");
    }

    private String buildScheduleHex(ScheduleSendVO.Schedule schedule) {
        String scheduleNo = schedule.getScheduleNo();
        Integer planNo = 0;
        String weekHex = "00";
        List<ScheduleSendVO.Schedule.Week> weeks = schedule.getWeeks();
        if (!CollectionUtils.isEmpty(weeks)) {
            List<ScheduleSendVO.Schedule.Week> weekSort = weeks.stream().sorted(Comparator.comparingInt(ScheduleSendVO.Schedule.Week::getWeekNum)).collect(Collectors.toList());
            int[] weekArray = new int[7];
            for (ScheduleSendVO.Schedule.Week week : weekSort) {
                Integer weekNum = week.getWeekNum();
                weekArray[weekNum - 1] = 1;
                planNo = week.getPlanNo();
            }
            weekHex = CommonUtils.getHex4Array(weekArray);
        }
        String monthHex = "0000";
        String dayHex = "00000000";
        List<ScheduleSendVO.Schedule.SpecialDay> specialDays = schedule.getSpecialDays();
        if (!CollectionUtils.isEmpty(specialDays)) {
            int[] monthArray = new int[12];
            int[] dayArray = new int[31];
            for (ScheduleSendVO.Schedule.SpecialDay specialDay : specialDays) {
                String dateStr = specialDay.getDateStr();
                monthArray[Integer.valueOf(dateStr.substring(5, 7)) - 1] = 1;
                dayArray[Integer.valueOf(dateStr.substring(8, 10)) - 1] = 1;
            }
            monthHex = CommonUtils.getHex4Array(monthArray);
            dayHex = CommonUtils.getHex4Array(dayArray);
        }
        String matchStr = "0c02%s%s";
        StringBuffer sb = new StringBuffer();
        for (int i = 2; i < 8; i++) { // 数据对象从2-7
            sb.append(String.format(matchStr, String.format("%02x", i), String.format("%02x", Integer.valueOf(scheduleNo))));
            if (i == 2) {
                sb.append("01");// 一代多路口编号
                sb.append(String.format("%02x", i)).append("05");
            }
            if (i == 3) {
                sb.append("00"); // 优先级
                sb.append(String.format("%02x", i)).append("05");
            }
            if (i == 4) {
                sb.append(weekHex); // 星期二进制转16
                sb.append(String.format("%02x", i)).append("06");
            }
            if (i == 5) {
                sb.append(StringUtils.leftPad(monthHex, 4, "0"));
                sb.append(String.format("%02x", i)).append("08");
            }
            if (i == 6) {
                sb.append(StringUtils.leftPad(dayHex, 8, "0"));
                sb.append(String.format("%02x", i)).append("05");
            }
            if (i == 7) {
                sb.append(String.format("%02x", planNo));
                sb.append(String.format("%02x", i)).append("05");
            }
        }
        return sb.toString().substring(0, sb.length() - 4);
    }

    @Override
    public JsonViewObject lockControl(ControlCommandVO commandVO) throws Exception {
        if (commandVO.getCommand() == 0) {
            setSignalControl(commandVO);
        } else {
            List<Integer> phaseList = commandVO.getPhaseList();
            int stageId = 0;
            if (!CollectionUtils.isEmpty(phaseList)) {
                stageId = phaseList.get(0);
            }  // 锁定阶段，目前锁定成功，但是阶段还在跳
            CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(commandVO.getCrossCode());
            String message = String.format(Constants.COMMAND_CENTER_COMMON, Constants.COMMAND_SET, Constants.COMMAND_CENTER_CONTROL_STAGE, String.format("%02x", stageId));
            String lockSign = StringUtils.join(Constants.COMMAND_CENTER_COMMON_SIGN, Constants.COMMAND_SET_REPLY);
            String lockMessage = Constants.buildMessage(message);
            MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), lockMessage, lockSign, 300);
            if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
                log.info("锁定相位成功");
            }
        }

        return JsonViewObject.newInstance().success("锁定相位成功");
    }

    @Override
    public JsonViewObject stepControl(String code, Integer command, Integer stepNum) throws Exception {
        return null;
    }

    @Override
    public JsonViewObject setSignalControl(ControlCommandVO commandVO) throws Exception {
        String controlmodel = null;
        if (commandVO.getCommand() == 0) {
            controlmodel = ControlModelEnum.FIXED_CYCLE.getDtControlHex(); // 取消控制，设置为固定配时
        } else {
            if (commandVO.getControlType() == 1) { // 全红
                controlmodel = ControlModelEnum.RED_CONTROL.getDtControlHex();
            }
            if (commandVO.getControlType() == 2) { // 黄闪
                controlmodel = ControlModelEnum.YELLOW_CONTROL.getDtControlHex();
            }
            if (commandVO.getControlType() == 3) { // 关灯
                controlmodel = ControlModelEnum.CLOSED_CONTROL.getDtControlHex();
            }
        }
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(commandVO.getCrossCode());
        String message = String.format(Constants.COMMAND_CENTER_COMMON, Constants.COMMAND_SET, Constants.COMMAND_CENTER_CONTROL_MODEL, controlmodel);
        String controlSign = StringUtils.join(Constants.COMMAND_CENTER_COMMON_SIGN, Constants.COMMAND_SET_REPLY);
        String controlMessageHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), controlMessageHex, controlSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            log.info("特殊控制成功，当前控制模式： {}", controlmodel);
        }
        return JsonViewObject.newInstance().success("特殊控制成功");
    }

    @Override
    public JsonViewObject recoverSchedule(String code) throws Exception {
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(code);
        String message = String.format(Constants.COMMAND_CENTER_COMMON, Constants.COMMAND_SET, Constants.COMMAND_CENTER_CONTROL_MODEL, ControlModelEnum.FIXED_CYCLE.getDtControlHex());
        String controlSign = StringUtils.join(Constants.COMMAND_CENTER_COMMON_SIGN, Constants.COMMAND_SET_REPLY);
        String controlMessageHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), controlMessageHex, controlSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            log.info("恢复时间表");
        }
        return JsonViewObject.newInstance().success("恢复时间表");
    }

    @Override
    public JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        String crossId = phaseTimingSendVO.getCrossCode();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        List<PhaseTimingSendVO.Phase> phaseList = phaseTimingSendVO.getPhaseList();
        setLights();
        setPhaseLights();
        setPhaseInfo(crossInfoPO, phaseList);
        return null;
    }

    private void setPhaseLights() {

    }

    private void setLights() {
    }

    private void setPhaseInfo(CrossInfoPO crossInfoPO, List<PhaseTimingSendVO.Phase> phaseList) {
        for (PhaseTimingSendVO.Phase phase : phaseList) {
            Integer phaseNo = phase.getPhaseNo();
            Integer yellow = phase.getYellow();
            Integer allRed = phase.getAllred();
            String greenFlash = phase.getGreenFlash();
            String maxGreen = phase.getMaxGreen();
            String minGreen = phase.getMinGreen();
            List<PhaseTimingSendVO.Phase.Lane> lanes = phase.getLanes();

            String matchStr = "0402%s%s";
            StringBuffer sb = new StringBuffer();
            for (int i = 2; i <= 31 ; i++) {
                sb.append(String.format(matchStr, String.format("%02x", i), String.format("%02x", phaseNo)));
                if (i == 2) {
                    sb.append("");
                }
            }

        }
    }

    @Override
    public JsonViewObject phaseDiffSend() {
        return null;
    }
}
